'use strict'

import React, { useState } from 'react'; 

import {
    PanelBody,
    PanelRow,
    TextareaControl,
} from "@wordpress/components";

import {
  useBlockProps,
  BlockControls,
  InnerBlocks,
  InspectorControls,
  RichText,
} from "@wordpress/block-editor";

// drupal import is set as external in webpack.config.js
import { t } from "drupal";
import RenderOpenAIResponse from "./render";
import parseOpenAIResponse from "./parser";

// i18n package could be also used. It is a wrapper around Drupal.t.
//   import { __ } from "@wordpress/i18n";
//   __('Text to be translated');

const ALLOWED_BLOCKS = ["core/heading", "core/paragraph", "core/quote"];

function Edit({ attributes, setAttributes }) {
  const { title, blockTitle, openaiAnswer } = attributes;
  const [ token, setToken ] = useState({token: ''});
  const [ isDisabled, setIsDisabled ] = useState(false);
  const [ processingMessage, setProcessingMessage ] = useState("");
  const [ questionCopy, setQuestionCopy ] = useState("");

  const handleClick = () => {
    /* Don't allow the user to click to request AI to answer the same question
     * many times, this sends unnecessary requests and consume extra pennies
     * that aren't needed */
    if (questionCopy === title){
      setProcessingMessage("You are asking the same question, please ask another question....");
      return false;
    }

    let data = {
      openai_prompt: title,
    };

    setIsDisabled(true);
    setProcessingMessage("Processing....");

    /* TODO: these fetch chains look ugly as hell, consider using async/await instead. */
    fetch('/session/token')
    .then(response => response.text())
    .then((csrfToken) => {
      fetch("/gutenberg-ai-tools/openai-rest?_format=json", {
        body: JSON.stringify(data),
        headers: {
          "Content-Type": "application/json",
          "X-Csrf-Token": csrfToken,
        },
        method: "POST"
      })
      .then(response => response.text())
      .then(data => {
        let parsed_data = parseOpenAIResponse(data);
        setToken({ token: parsed_data });
        setAttributes({ openaiAnswer: parsed_data });
        setIsDisabled(false);
        setProcessingMessage("");
        setQuestionCopy(title);
    })
    .catch(error => console.error(error));
    });
  }

  const postSearch = (value) => {
    setAttributes({ openaiAnswer: value });
    setIsDisabled(false);
    setProcessingMessage("");
    setQuestionCopy(title);
  }

  return (
    /* eslint-disable-next-line react/jsx-props-no-spreading */
    <div {...useBlockProps()}>
      <BlockControls>
      </BlockControls>
      <InspectorControls>
        <PanelBody title={ t('Block settings') } initialOpen>
          <PanelRow>
          </PanelRow>
          <PanelRow>
            <TextareaControl
              label={t('Block Title')}
              help={t('Use this field if you want to put a question title above the answer')}
              value={blockTitle}
              onChange={(value) => setAttributes({ blockTitle: value })}
            />
          </PanelRow>
          <PanelRow>
            <TextareaControl
              label={t('OpenAI Answer')}
              help={t('OpenAI answer generated by the engine. You can change the answer here')}
              value={openaiAnswer}
              onChange={(value) => postSearch(value)}
            />
          </PanelRow>
        </PanelBody>
      </InspectorControls>
      <div>
        <RichText
          tagName="h2"
          placeholder={t("Start typing your question here:")}
          value={title}
          onChange={(value) => setAttributes({ title: value })}
        />
        <button onClick={handleClick} disabled={isDisabled}>{t('Ask AI')}</button><span>&nbsp;{processingMessage}</span>
        <div>
          <h3>{blockTitle}</h3>
          <RenderOpenAIResponse question={title} block_title={blockTitle} answer={openaiAnswer} />
          <InnerBlocks allowedBlocks={ALLOWED_BLOCKS} />
        </div>
      </div>
    </div>
  );
};

export default Edit;