<?php

namespace Drupal\gutenberg_ai_tools\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\ai\AiProviderPluginManager;

/**
 * Class GutenbergAISettings.
 *
 * Provides configuration settings for the Gutenberg AI Plugin
 */
class GutenbergAISettings extends ConfigFormBase {

  /**
   * The AI provider plugin manager.
   *
   * @var \Drupal\ai\AiProviderPluginManager
   */
  protected $aiProvider;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'gutenberg_ai_tools_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['gutenberg_ai_tools.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->aiProvider = $container->get('ai.provider');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $config = $this->config('gutenberg_ai_tools.settings');

    $form['options'] = [
      '#type' => 'markup',
      '#markup' => '<p>Set the AI options for the response sent to Gutenberg AI Block, here you can set what model to use, temperature options and tokens.</p><br />',
    ];

    // Use 'chat' operation type as per AI module documentation.
    $operation_type = 'chat';

    // Check if we have any providers for this operation type.
    if (!$this->aiProvider->hasProvidersForOperationType($operation_type, TRUE)) {
      $form['no_providers'] = [
        '#type' => 'markup',
        '#markup' => $this->t('No AI providers available for chat. Please configure at least one provider in the <a href="@url">AI module settings</a>.', ['@url' => '/admin/config/ai']),
      ];

      return $form;
    }

    // Get the simple provider/model options as recommended in the documentation.
    $provider_model_options = $this->aiProvider->getSimpleProviderModelOptions($operation_type);

    if (empty($provider_model_options)) {
      $form['no_models'] = [
        '#type' => 'markup',
        '#markup' => $this->t('No models available for chat. Please configure at least one provider with chat models in the <a href="@url">AI module settings</a>.', ['@url' => '/admin/config/ai']),
      ];
      return $form;
    }

    $form['provider_model'] = [
      '#type' => 'select',
      '#title' => $this->t('AI Provider and Model'),
      '#options' => $provider_model_options,
      '#default_value' => $config->get('provider_model'),
      '#description' => $this->t('Select which AI provider and model to use.'),
    ];

    $form['temperature'] = [
      '#min' => 0,
      '#max' => 2,
      '#step' => .1,
      '#default_value' => $config->get("temperature") ?: 0.7,
      '#description' => $this->t('What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.'),
    ];


    $form['max_tokens'] = [
      '#type' => 'number',
      '#title' => $this->t('Max tokens'),
      '#min' => 128,
      '#step' => 1,
      '#default_value' => $config->get("max_tokens") ?: 1024,
      '#description' => $this->t("The maximum number of tokens to generate in the completion. The token count of your prompt plus max_tokens cannot exceed the model's context length."),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('gutenberg_ai_tools.settings');

    $max_tokens = (int) $form_state->getValue('max_tokens');
    $provider_model = $form_state->getValue('provider_model');
    $temperature = $form_state->getValue('temperature');

    $config->set('provider_model', $provider_model)->save();
    $config->set('max_tokens', $max_tokens)->save();
    $config->set('temperature', $temperature)->save();

    parent::submitForm($form, $form_state);
  }

}
